/**
 * Copyright [2024] Jiří Štefka <jiriks74>
 * Project: AdventOfCode
 * @file main.c
 * @brief Main entry point
 * @author jiriks74
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define INC_CHUNK = 64

int compareInt(const void *a, const void *b) {
  int int_a = *((int *)a);
  int int_b = *((int *)b);
  return int_a < int_b ? -1 : int_a == int_b ? 0 : 1;
}

int compare(const void *a, const void *b) {
  int int_a = *((int *)a);
  int int_b = *((int *)b);

  if (int_a == int_b)
    return 0;
  else if (int_a < int_b)
    return -1;
  else
    return 1;
}

/**
 * @brief Main entry point
 * @param argc Number of command-line arguments.
 * @param argv Array of command-line arguments.
 */
#ifndef TESTING
int main(int argc, char *argv[])
#endif
#ifdef TESTING
    int main_test(int argc, char *argv[])
#endif
{

  FILE *file;
  if (argc == 1) {
    file = stdin;
  } else {
    file = fopen(argv[1], "r");
  }

  if (file == NULL) {
    if (argc > 1)
      fprintf(stderr, "Could not open file '%s'\n", argv[1]);
    else
      fprintf(stderr, "Couldn't open file\n");
    exit(EXIT_FAILURE);
  }

  char *buffer = NULL;
  size_t bufferSize = 0;
  uint lines = 0;
  while (getline(&buffer, &bufferSize, file) != -1) {
    lines++;
  }

  int array1[lines];
  int array2[lines];

  rewind(file);

  for (uint i = 0; i < lines; i++) {
    if (getline(&buffer, &bufferSize, file) == -1)
      break;

    if(strcmp(buffer, "")) continue;

    sscanf(buffer, "%d   %d", &array1[i], &array2[i]);
  }

  free(buffer);
  bufferSize = 0;
  fclose(file);

  qsort(array1, lines, sizeof(int), compareInt);
  qsort(array2, lines, sizeof(int), compareInt);

  uint distanceSum = 0;
  for(uint i = 0; i < lines; i++){
    distanceSum += array1[i] < array2[i] ? array2[i] - array1[i] : array1[i] - array2[i];
  }

  uint reoccuranceSum = 0;
  for(uint i = 0; i < lines; i++){
    uint reoccurance = 0;
    for(uint j = 0; j < lines; j++){
      if(array1[i] == array2[j]) reoccurance++;
    }
    reoccuranceSum += array1[i] * reoccurance;
  }

  printf("The distance between my lists is %d\n", distanceSum);
  printf("The simmilarity score is %d\n", reoccuranceSum);

  return 0;
}
